from django.conf.urls import url
from .views import (
    new_session, del_session, all_sessions, edit_categories, add_category,
    edit_category, delete_category, reorder_categories, add_item,
    edit_item, delete_item, reorder_items, session, save_magnet,
    delete_magnet,
)

urlpatterns = [
    url(r'^new_session/$', new_session, {}, 'fridge-new-session'),
    url(r'^del_session/(?P<id>\d+)/$', del_session, {}, 'fridge-del-session'),
    url(r'^all_sessions/$', all_sessions, {}, 'fridge-all-sessions'),
    url(r'^(?P<id>\d+)/edit_categories/$', edit_categories, {},
        'fridge-edit-categories'),

    url(r'^(?P<id>\d+)/edit_categories/add_category/$', add_category, {},
        'edit-fridge-add-category'),
    url(r'^(?P<id>\d+)/edit_category/$', edit_category, {},
        'edit-fridge-category'),
    url(r'^(?P<id>\d+)/delete_category/$', delete_category, {},
        'delete-fridge-category'),
    url(r'^(?P<id>\d+)/reorder_categories/$', reorder_categories, {},
        'reorder-fridge-categories'),

    url(r'^(?P<id>\d+)/edit_category/add_item/$', add_item, {},
        'edit-fridge-add-item'),
    url(r'^(?P<id>\d+)/edit_item/$', edit_item, {}, 'edit-fridge-item'),
    url(r'^(?P<id>\d+)/delete_item/$', delete_item, {}, 'delete-fridge-item'),
    url(r'^(?P<id>\d+)/reorder_items/$', reorder_items, {},
        'reorder-fridge-items'),

    url(r'^session/(?P<id>\d+)/$', session, {}, 'fridge-session'),
    url(r'^session/(?P<id>\d+)/save_magnet/$', save_magnet, {},
        'fridge-save-magnet'),
    url(r'^session/(?P<id>\d+)/delete_magnet/$', delete_magnet, {},
        'fridge-delete-magnet'),
]



from django import template
from fridgeblock.models import Magnet,Session

register = template.Library()

class MagnetExistsForSessionNode(template.Node):
    def __init__(self, session, item, nodelist_true, nodelist_false=None):
        self.nodelist_true = nodelist_true
        self.nodelist_false = nodelist_false
        self.session = session
        self.item = item

    def render(self, context):
        if self.session not in context:
            return self.nodelist_false.render(context)
        s = context[self.session]
        i = context[self.item]
        if Magnet.objects.filter(session=s,item=i).count():
            return self.nodelist_true.render(context)
        else:
            return self.nodelist_false.render(context)

@register.tag('if_magnet_for_item_exists')
def accessible(parser, token):
    session = token.split_contents()[1:][0]
    item    = token.split_contents()[1:][1]
    nodelist_true = parser.parse(('else','endif_magnet_for_item_exists'))
    token = parser.next_token()
    if token.contents == 'else':
        nodelist_false = parser.parse(('endif_magnet_for_item_exists',))
        parser.delete_first_token()
    else:
        nodelist_false = None
    return MagnetExistsForSessionNode(session, item, nodelist_true, nodelist_false)


class GetSessionNode(template.Node):
    def __init__(self,fridge,var_name):
        self.fridge = fridge
        self.var_name = var_name

    def render(self, context):
        f = context[self.fridge]
        u = context['request'].user
        if u.is_anonymous():
            return ''
        r = Session.objects.filter(fridgeblock=f,user=u).order_by("-saved")
        if r.count() == 0:
            context[self.var_name] = Session.objects.create(fridgeblock=f,user=u)
        else:
            context[self.var_name] = r[0]
        return ''

@register.tag('getsession')
def getquestionresponse(parser, token):
    fridge = token.split_contents()[1:][0]
    var_name = token.split_contents()[1:][2]
    return GetSessionNode(fridge,var_name)



# Create your views here.
from django.contrib.auth.decorators import permission_required
from django.contrib.auth.decorators import login_required
from django.template import RequestContext
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import render_to_response, get_object_or_404
from models import *
from django.core.urlresolvers import reverse
from pagetree.helpers import get_hierarchy

class rendered_with(object):
    def __init__(self, template_name):
        self.template_name = template_name

    def __call__(self, func):
        def rendered_func(request, *args, **kwargs):
            items = func(request, *args, **kwargs)
            if type(items) == type({}):
                return render_to_response(self.template_name, items, context_instance=RequestContext(request))
            else:
                return items

        return rendered_func

@login_required
def index(request):
    user = request.user
    # let's just send them to the most recent session
    newest = Session.objects.filter(user=user).order_by("-id")
    if newest.count() == 0:
        # create one
        return HttpResponseRedirect(reverse('fridge-new-session'))
    else:
        return HttpResponseRedirect(reverse('fridge-session',args=[newest[0].id]))


@login_required
@rendered_with('healthhabitplan/session.html')
def session(request,id):
    s = get_object_or_404(Session,id=id)
    h = get_hierarchy()
    return dict(session=s,
                sessions=Session.objects.filter(user=request.user),
                categories=Category.objects.all(),
                root = h.get_root()
                )

@login_required
@rendered_with('healthhabitplan/all_sessions.html')
def all_sessions(request):
    user = request.user
    h = get_hierarchy()
    sessions = Session.objects.filter(user=request.user)
    return dict(sessions=sessions, categories=Category.objects.all(), root = h.get_root())

@login_required
def new_session(request):
    user = request.user
    s = Session.objects.create(user=user)
    return HttpResponseRedirect(reverse('fridge-session',args=[s.id]))

@login_required
def del_session(request, id=id):
    user = request.user
    s = Session.objects.get(id=id)
    s.delete()
    return HttpResponseRedirect(reverse('fridge-index'))


@login_required
def save_magnet(request,id):
    s = get_object_or_404(Session,id=id)
    if request.method == "POST":
        item_id = request.GET['item_id']
        item = get_object_or_404(Item,id=item_id)
        x = request.GET['x']
        y = request.GET['y']
        r = Magnet.objects.filter(session=s,item=item)
        if r.count() > 0:
            m = r[0]
            m.x = x
            m.y = y
            m.save()
        else:
            m = Magnet.objects.create(session=s,item=item,x=x,y=y)
    return HttpResponse("ok");

@login_required
def delete_magnet(request,id):
    s = get_object_or_404(Session,id=id)
    if request.method == "POST":
        item_id = request.GET['item_id']
        item = get_object_or_404(Item,id=item_id)
        r = Magnet.objects.filter(session=s,item=item)
        if r.count() > 0:
            m = r[0]
            m.delete()
    return HttpResponse("ok");

@login_required
@rendered_with("fridgeblock/edit_categories.html")
def edit_categories(request,id):
    block = get_object_or_404(FridgeBlock,id=id)
    return dict(fridge=block)

def reorder_categories(request,id):
    if request.method != "POST":
        return HttpResponse("only use POST for this", status=400)
    fridge = get_object_or_404(FridgeBlock,id=id)
    keys = request.GET.keys()
    category_keys = [int(k[len('category_'):]) for k in keys if k.startswith('category_')]
    category_keys.sort()
    categories = [int(request.GET['category_' + str(k)]) for k in category_keys]
    fridge.set_category_order(categories)
    return HttpResponse("ok")

def delete_category(request,id):
    c = get_object_or_404(Category,id=id)
    fridge = c.fridgeblock
    c.delete()
    return HttpResponseRedirect(reverse("fridge-edit-categories",args=[fridge.id]))

@rendered_with('fridgeblock/edit_category.html')
def edit_category(request,id):
    category = get_object_or_404(Category,id=id)
    if request.method == "POST":
        form = category.edit_form(request.POST)
        category = form.save(commit=False)
        category.save()
        return HttpResponseRedirect(reverse("edit-fridge-category",args=[category.id]))
    return dict(category=category)


def add_category(request,id):
    fridge = get_object_or_404(FridgeBlock,id=id)
    form = fridge.add_category_form(request.POST,request.FILES)
    if form.is_valid():
        layer = form.save(commit=False)
        layer.fridgeblock = fridge
        layer.save()
    else:
        print "form was not valid"
        print form.errors
    return HttpResponseRedirect(reverse("fridge-edit-categories",args=[fridge.id]))



def reorder_items(request,id):
    if request.method != "POST":
        return HttpResponse("only use POST for this", status=400)
    category = get_object_or_404(Category,id=id)
    keys = request.GET.keys()
    item_keys = [int(k[len('item_'):]) for k in keys if k.startswith('item_')]
    item_keys.sort()
    items = [int(request.GET['item_' + str(k)]) for k in item_keys]
    category.set_item_order(items)
    return HttpResponse("ok")

def delete_item(request,id):
    c = get_object_or_404(Item,id=id)
    category = c.category
    c.delete()
    return HttpResponseRedirect(reverse("edit-fridge-category",args=[category.id]))

@rendered_with('fridgeblock/edit_item.html')
def edit_item(request,id):
    item = get_object_or_404(Item,id=id)
    if request.method == "POST":
        form = item.edit_form(request.POST)
        item = form.save(commit=False)
        item.save()
        return HttpResponseRedirect(reverse("fridge-edit-item",args=[item.id]))
    return dict(item=item)


def add_item(request,id):
    category = get_object_or_404(Category,id=id)
    form = category.add_item_form(request.POST)
    if form.is_valid():
        item = form.save(commit=False)
        item.category = category
        item.save()
    else:
        print "form was not valid"
        print form.errors
    return HttpResponseRedirect(reverse("edit-fridge-category",args=[category.id]))




from django.db import models
from pagetree.models import PageBlock
from django.contrib.auth.models import User
from django.contrib.contenttypes.fields import GenericRelation
from django import forms
from datetime import datetime
from django.core.urlresolvers import reverse
from django.template.defaultfilters import slugify
from django.conf import settings
from sorl.thumbnail.fields import ImageWithThumbnailsField
import os

class FridgeBlock(models.Model):
    pageblocks = GenericRelation(PageBlock)
    template_file = "fridgeblock/fridgeblock.html"
    js_template_file = "fridgeblock/fridgeblock_js.html"
    css_template_file = "fridgeblock/fridgeblock_css.html"

    display_name = "Fridge Block"
    
    def pageblock(self):
        return self.pageblocks.all()[0]

    def __unicode__(self):
        return unicode(self.pageblock())

    def edit_form(self):
        class EditForm(forms.Form):
            alt_text = "<a href=\"" + reverse("fridge-edit-categories",args=[self.id]) + "\">categories</a><br />"
        return EditForm()
    
    def edit(self,vals,files=None):
        self.save()

    @classmethod
    def add_form(self):
        class AddForm(forms.Form):
            pass
        return AddForm()

    @classmethod
    def create(self,request):
        return FridgeBlock.objects.create()

    def needs_submit(self):
        return False

    def categories(self):
        return self.category_set.all()

    def add_category_form(self,request=None,files=None):
        return CategoryForm(request,files)

    @classmethod
    def get_exporter(self,export_type):
        e = Exporter()
        return (e.identifier,e.exporter)

class Exporter(object):
    block_class = FridgeBlock
    identifier = 'fridge'

    def exporter(self, block, xmlfile, zipfile):
        pass

    def importer(self, node, zipfile):
        return None


class Category(models.Model):
    fridgeblock = models.ForeignKey(FridgeBlock)
    label = models.CharField(max_length=256,default="")
    image = ImageWithThumbnailsField(upload_to="images/fridge/categories/%Y/%m/%d",
                                     thumbnail = {
            'size' : (65,65)
            },
                                     extra_thumbnails={
            'admin': {
                'size': (70, 50),
                'options': ('sharpen',),
                }
            })


    class Meta:
        order_with_respect_to = "fridgeblock"

    def __unicode__(self):
        return self.label

    def css(self):
        return slugify(self.label)

    def edit_form(self,request=None):
        return CategoryForm(request,instance=self)

    def edit(self,vals,files):
        self.label = vals.get('label','')
        if 'image' in files:
            self.save_image(files['image'])
        self.save()

    def add_item_form(self,request=None):
        return ItemForm(request)

    def save_image(self,f):
        ext = f.name.split(".")[-1].lower()
        basename = slugify(f.name.split(".")[-2].lower())[:20]
        if ext not in ['jpg','jpeg','gif','png']:
            # unsupported image format
            return None
        now = datetime.now()
        path = "images/fridge/categories/%04d/%02d/%02d/" % (now.year,now.month,now.day)
        try:
            os.makedirs(settings.MEDIA_ROOT + "/" + path)
        except:
            pass
        full_filename = path + "%s.%s" % (basename,ext)
        fd = open(settings.MEDIA_ROOT + "/" + full_filename,'wb')
        for chunk in f.chunks():
            fd.write(chunk)
        fd.close()
        self.image = full_filename
        self.save()



class CategoryForm(forms.ModelForm):
    class Meta:
        model = Category
        exclude = ("fridgeblock",)

class Item(models.Model):
    label = models.CharField(max_length=256,default="")
    category = models.ForeignKey(Category)
    description = models.TextField(default="",blank=True)

    class Meta:
        order_with_respect_to = 'category'

    def __unicode__(self):
        return self.label

    def edit_form(self,request=None):
        return ItemForm(request,instance=self)

class ItemForm(forms.ModelForm):
    class Meta:
        model = Item
        exclude = ("category",)

class Session(models.Model):
    fridgeblock = models.ForeignKey(FridgeBlock)
    user = models.ForeignKey(User)
    saved = models.DateTimeField(auto_now=True)

    class Meta:
        ordering = ("user","saved")

    def __unicode__(self):
        return "session #%d for %s %s" % (self.number(),self.user.first_name, self.user.last_name)

    def number(self):
        r = Session.objects.filter(user=self.user,fridgeblock=self.fridgeblock,
                                   saved__lt=self.saved).count()
        return r + 1

class Magnet(models.Model):
    session = models.ForeignKey(Session)
    item = models.ForeignKey(Item)

    x = models.IntegerField(default=0)
    y = models.IntegerField(default=0)


